#ifndef GRAPH_H
#define GRAPH_H

#include <QDialog>
#include <QTimer>
#include "liveboxinfos.h"
#include "qwt_plot.h"
#include "qwt_plot_curve.h"
#include "qwt_scale_engine.h"
#include "qwt_date_scale_engine.h"
#include "qwt_scale_draw.h"
#include "qwt_date_scale_draw.h"
#include "qwt_plot_picker.h"
#include "qwt_plot_marker.h"
#include "qwt_date.h"
#include "qwt_plot_grid.h"
#include "qwt_plot_renderer.h"
#include "qwt_picker_machine.h"
#include "qwt_plot_zoneitem.h"

#define GRAPH_MARGES 0
#define GRAPH_MAX_RATE 1
#define GRAPH_SYNCHRO 2
#define GRAPH_CRC 0
#define GRAPH_HEC 1
#define GRAPH_FEC 2

namespace Ui {
class Graph;
}

class Graph : public QDialog
{
    Q_OBJECT

public:
    explicit Graph(QWidget *parent = 0);
    ~Graph();
    void setDiapo(unsigned int value)           {Diapo          = value;}
    void setExportFormat(QString value)         {ExportFormat   = value;}
    void setDiapoErreurs(bool value)            {DiapoErreurs   = value;}
    void setDiapoCourbes(bool value)            {DiapoCourbes   = value;}
    void setSSID(QString value)                 {SSID           = value;}
    void setDebut(double value)                 {Debut          = value;}
    void setTempsArret(double value)            {TempsArret     = value;}
    void setEchHeure(bool value)                {EchHeure       = value;}
    void setEchDate(bool value)                 {EchDate        = value;}
    void setRapPageOrient(int value)            {RapPageOrient  = value;}
    void setHardwareVersion(QString value)      {HardwareVersion= value;}
    void setSoftwareVersion(QString value)      {SoftwareVersion= value;}
    double getPlageHeure()                      {return PlageHeure;}

public slots:
    void UpDateCurves(LiveBoxInfos* LBox, LiveBoxInfos* OldLBox);
    void UpDateScales();
    void ClearCurves();
    void ResetMarker();
    void RapportGraphique(QString File, QString Day);
    void UpDateSynchronisations();
    void SetZoomTout();
    void SetZoom3H();
    void SetZoomPlage(double value);

private slots:
    void on_BtnDiapo_toggled(bool checked);
    void DiapoTimerHandler();
    void InitTabSynchronisations();

    void on_scrollTemps_valueChanged(int value);
    void on_checkAvanceAuto_toggled(bool checked);
    void on_BtnZoomPlus_clicked();
    void on_BtnZoomMoins_clicked();
    void on_BtnZoom3H_clicked();
    void on_BtnZoomTout_clicked();
    void on_BtnNoiseMaxPlus_clicked();
    void on_BtnNoiseMinMoins_clicked();
    void on_BtnNoiseMaxMoins_clicked();
    void on_BtnNoiseMinPlus_clicked();
    void on_BtnMaxRateMaxPlus_clicked();
    void on_BtnMaxRateMaxMoins_clicked();
    void on_BtnMaxRateMinPlus_clicked();
    void on_BtnMaxRateMinMoins_clicked();
    void on_BtnFECMaxPlus_clicked();
    void on_BtnFECMaxMoins_clicked();
    void on_BtnHECMaxPlus_clicked();
    void on_BtnHECMaxMoins_clicked();
    void on_BtnCRCMaxPlus_clicked();
    void on_BtnCRCMaxMoins_clicked();
    void on_BtnExportNoise_clicked();
    void on_BtnExportMaxRate_clicked();
    void on_BtnExportFEC_clicked();
    void on_BtnExportHEC_clicked();
    void on_BtnExportCRC_clicked();

    void on_tabCourbes_currentChanged();
    void on_tabErreurs_currentChanged();
    void on_Picker_Temps(QPointF Pos);

    void addZone(double Deb, double Fin);
    void clearListZone();

    void on_BtnSynchroPlus_clicked();

    void on_BtnSynchroMoins_clicked();

    void on_BtnExportSynchro_clicked();

    void on_BtnSynchroPdf_clicked();

    void on_BtnUpDateSynchro_clicked();

protected:
    bool eventFilter(QObject *obj, QEvent *ev);

private:

    Ui::Graph *ui;
    unsigned int    Diapo;
    QString         ExportFormat;
    QTimer         *DiapoTimer;

    // pour les échelles horizontales date et heure
    QwtDateScaleDraw *qwtScaleDrawNoise;
    QwtDateScaleDraw *qwtScaleDrawMaxRate;
    QwtDateScaleDraw *qwtScaleDrawCRC;
    QwtDateScaleDraw *qwtScaleDrawHEC;
    QwtDateScaleDraw *qwtScaleDrawFEC;
    // pour les tracés de courbes avec Qwt
    QwtPlotCurve   *CurveUpNoise;
    QPolygonF       PointUpNoise;
    QwtPlotCurve   *CurveDownNoise;
    QPolygonF       PointDownNoise;
    QwtPlotCurve   *CurveUpMaxRate;
    QPolygonF       PointUpMaxRate;
    QwtPlotCurve   *CurveDownMaxRate;
    QPolygonF       PointDownMaxRate;
    QwtPlotCurve   *CurveUpDslRate;
    QPolygonF       PointUpDslRate;
    QwtPlotCurve   *CurveDownDslRate;
    QPolygonF       PointDownDslRate;
    QwtPlotCurve   *CurveLogCRC;
    QPolygonF       PointLogCRC;
    QwtPlotCurve   *CurveLogATUCCRC;
    QPolygonF       PointLogATUCCRC;
    QwtPlotCurve   *CurveLogHEC;
    QPolygonF       PointLogHEC;
    QwtPlotCurve   *CurveLogATUCHEC;
    QPolygonF       PointLogATUCHEC;
    QwtPlotCurve   *CurveLogFEC;
    QPolygonF       PointLogFEC;
    QwtPlotCurve   *CurveLogATUCFEC;
    QPolygonF       PointLogATUCFEC;
    // Pour les marques de pertes de synchro
    QwtPlotCurve   *CurveSynchNoise;
    QwtPlotCurve   *CurveSynchMaxRate;
    QwtPlotCurve   *CurveSynchCRC;
    QwtPlotCurve   *CurveSynchHEC;
    QwtPlotCurve   *CurveSynchFEC;
    QPolygonF       PointSynch;
    // pour les grilles de graduations
    QwtPlotGrid    *GridNoise;
    QwtPlotGrid    *GridMaxRate;
    QwtPlotGrid    *GridCRC;
    QwtPlotGrid    *GridHEC;
    QwtPlotGrid    *GridFEC;
    // pour cliquer sur un des graphiques et reporter la marque sur les autres
    QwtPlotPicker*  PickerNoise;
    QwtPlotPicker*  PickerMaxRate;
    QwtPlotPicker*  PickerCRC;
    QwtPlotPicker*  PickerHEC;
    QwtPlotPicker*  PickerFEC;
    QwtPlotMarker   MarkerNoise;
    QwtPlotMarker   MarkerMaxRate;
    QwtPlotMarker   MarkerCRC;
    QwtPlotMarker   MarkerHEC;
    QwtPlotMarker   MarkerFEC;
    // pour le graphique de synthese des synchronisations
    QwtDateScaleDraw *qwtScaleXsynchro;     // pour les heures en X (de 00:00 à 23:59)
    QwtDateScaleDraw *qwtScaleYsynchro;     // pour les jours en Y
    QwtPlotCurve     *CurveSynchronisations;
    QPolygonF         PointSynchronisations;
    QwtPlotGrid      *GridSynchronisations;
    QwtPlotCurve     *CurveReboot;
    QPolygonF         PointReboot;
    QwtPlotCurve     *CurveConnect;
    QPolygonF         PointConnect;

    //  pour le marquage de zones telles que l'arrêt d'enregistrement
    QList<QwtPlotZoneItem*> ListZone;
    double          TempsArret;         // facteur d'arrêt, un arrêt de relevé est marqué si le temps dépasse FactArret secondes

    double          Debut;              // début d'enregistrement (heure recalculée selon Qwt)
    double          Dernier;            // dernier enregistrement (heure recalculée selon Qwt)
    double          BordDroit;          // heure de relevé + décalage (ReculMinute)
    double          Offset;             // décalage donné par le slider de temps
    double          TpsMajor;           // graduation majeur de temps (par défaut 30mn)
    int             NbTpsMin;           // nombre de graduations mineures (par défaut 6 soit 5mn)
    int             NbTpsMed;           // nombre de graduations médiannes (par défaut 2 soit 15mn)

    double          MinNoise;
    double          MaxNoise;
    double          MinMaxRate;
    double          MaxMaxRate;
    double          PasMaxRate;
    double          MinCRC;
    double          MaxCRC;
    double          MinHEC;
    double          MaxHEC;
    double          MinFEC;
    double          MaxFEC;
    double          NbJrSynch;          // nombre de jours de recul sur l'analyse des synchros

    double          ReculMinute;        // décalage de "bord de graphique" en minutes
    double          PlageHeure;         // nombre d'heures visible sur le graphique (heures)
    double          TempsMaxHeure;      // durée maxi d'un graphique
    double          DecHistoSec;        // pour ne pas superposer les barres d'un même relevé = décalage droite et gauche
    double          PenCurve;
    double          PenHisto;
    double          PenSynch;
    double          CoefPen;            // pour réduire les largeurs de trait en fonction de l'échelle de temps
    double          ExportScale;
    int             ExportResol;

    bool            DiapoErreurs;
    bool            DiapoCourbes;

    QString         SSID;
    QString         HardwareVersion;
    QString         SoftwareVersion;

    QString         TpsHeureFormat;
    QString         TpsDateFormat;
    QString         TpsSeparFormat;
    QString         TpsFormat;
    bool            EchHeure;
    bool            EchDate;
    int             RapPageOrient;      // portrait ou paysage

    double          XtpsMin;            // pour conserver les extrémités d'échelle de temps
    double          XtpsMax;
};

#endif // GRAPH_H
