#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QCoreApplication>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QUrl>
#include <QUrlQuery>
#include <QWebSettings>
#include <QVariant>
#include <QJsonValue>
#include <QJsonDocument>
#include <QJsonObject>
#include <QVariantMap>
#include <QJsonArray>
#include <QFile>
#include <QLabel>
#include <QSettings>
#include <QMessageBox>
#include <QPrinter>
#include <QListData>
#include <QDebug>
#include "liveboxinfos.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    WinAccueil  = new Accueil();
    Graphiques  = new Graph();
    WinAccueil->show();

    //ui->BtnRapportGraphique->setVisible(false);
    ui->btnReboot->setVisible(false);

    LastSyncho="";
    LastChangeTime=0;
    PppLastChangeTime=0;
    SessionDateTime = QDateTime::currentDateTime().toString("dd/MM/yyyy hh:mm:ss");
    AlgoType=0; //par défaut pour LB2 ou 3

    // etat des boutons au démarrage
    ui->BtnActualiser->setEnabled(true);
    ui->BtnGraphiques->setEnabled(true);
    ui->BtnStartAcquisition->setEnabled(true);
    ui->BtnStopAcquisition->setEnabled(false);
    ui->BtnSaveReleve->setEnabled(true);
    ui->BtnRazReleve->setEnabled(true);
    ui->BtnQuitter->setEnabled(true);
    ui->tabWidget->setCurrentIndex(CONNEXION);

    // renseignement de la barre de status
    QLabel* StatusLabel = new QLabel(this);
    StatusLabel->setTextFormat(Qt::RichText);
    StatusLabel->setText(QString(VERSION)+" - &#169; JLS-2015/2020 - <img src=:/icons/QtCreator height=\"16\" width=\"16\" > - Qt.5.5.0 / Qwt.6.1.2");
    ui->statusBar->addPermanentWidget(StatusLabel);

    // lecture des paramètres stockes dans "livebox.ini"
    QSettings settings(QApplication::applicationDirPath() + "/livebox.ini", QSettings::IniFormat);
    LiveBox         =settings.value("LIVEBOX/LiveBox").toString();
    User            =settings.value("LIVEBOX/User").toString();
    Pass            =settings.value("LIVEBOX/Pass").toString();
    SSID            =settings.value("LIVEBOX/SSID").toString();
    AlgoType        =settings.value("LIVEBOX/AlgoType").toInt();
    ui->editPass->setText(Pass);
    if (AlgoType==0)
        ui->radioLB23->setChecked(true);
    else
        ui->radioLB34->setChecked(true);
    QString sCycle  =settings.value("MONITOR/Cycle").toString();
    ui->timeEditCycle->setTime(QTime::fromString(sCycle,"hh:mm:ss"));
    Cycle=3600*QTime::fromString(sCycle).hour()+60*QTime::fromString(sCycle).minute()+QTime::fromString(sCycle).second();
    Cycle=1000*Cycle;               // le cycle dans un QTimer doit être donné en milli secondes.
    QString stArret  =settings.value("MONITOR/Arret").toString();
    ui->timeEditArret->setTime(QTime::fromString(stArret,"hh:mm:ss"));
    QTime tArret=QTime::fromString(ui->timeEditArret->text(),"hh:mm:ss");
    int sArret=3600*tArret.hour()+60*tArret.minute()+tArret.second();
    Graphiques->setTempsArret(sArret);

    QString sDiapo  =settings.value("MONITOR/Diapo").toString();
    ui->timeEditDiapo->setTime(QTime::fromString(sDiapo,"hh:mm:ss"));
    Diapo=3600*QTime::fromString(sDiapo).hour()+60*QTime::fromString(sDiapo).minute()+QTime::fromString(sDiapo).second();
    Diapo=1000*Diapo;               // le cycle dans un QTimer doit être donné en milli secondes.
    Graphiques->setDiapo(Diapo);    // on envoie la valeur au graphique
    ui->comboExportFormat->setCurrentIndex(settings.value("MONITOR/ExportFormat").toInt());
    Graphiques->setExportFormat(ui->comboExportFormat->currentText());
    ui->checkDiapoCourbes->setChecked(settings.value("COURBES/DiapoCourbes").toBool());
    Graphiques->setDiapoCourbes(ui->checkDiapoCourbes->isChecked());
    ui->checkDiapoErreurs->setChecked(settings.value("COURBES/DiapoErreurs").toBool());
    Graphiques->setDiapoErreurs(ui->checkDiapoErreurs->isChecked());
    ui->checkAffDate->setChecked(settings.value("FORMAT/EchDate").toBool());
    ui->checkAffHeure->setChecked(settings.value("FORMAT/EchHeure").toBool());
    ui->checkReloadCSV->setChecked(settings.value("MONITOR/ReloadCSV").toBool());
    ui->checkSaveAuto->setChecked(settings.value("MONITOR/SaveAuto").toBool());
    ui->checkRazAuto->setChecked(settings.value("MONITOR/RazAuto").toBool());
    ui->checkPdfAuto->setChecked(settings.value("MONITOR/PdfAuto").toBool());
    if (settings.value("FORMAT/RapPageOrient").toInt()==QPrinter::Landscape) ui->checkPaysage->setChecked(true);
    if (settings.value("FORMAT/RapPageOrient").toInt()==QPrinter::Portrait) ui->checkPortrait->setChecked(true);

    // initialisation des URL et paramètres de communication avec la LiveBox
    URLgetMIBs          =QString(URL_get_MIBs);
    URLgetPPP           =QString(URL_get_PPP);
    URLgetDSLStats      =QString(URL_get_DSL_Stats);
    URLgetWIFI          =QString(URL_get_WIFI);
    URLgetDevice        =QString(URL_get_Device);
    URLgetWANstatus     =QString(URL_get_WAN_status);
    PARAMSgetMIBs       =QByteArray(PARAMS_get_MIBs);
    PARAMSgetPPP        =QByteArray(PARAMS_get_PPP);
    PARAMSgetDSLStats   =QByteArray(PARAMS_get_DSL_Stats);
    PARAMSgetWififlags  =QByteArray(PARAMS_get_Wifi_flags);
    PARAMSgetWANstatus  =QByteArray(PARAMS_get_WAN_status);

    // remet les dimensions et position de fenetre de la dernière utilisation
    this->setGeometry(settings.value("GUI/X").toInt(),settings.value("GUI/Y").toInt(),settings.value("GUI/Width").toInt(),settings.value("GUI/Height").toInt());

    // choix de l'onglet actif
    ui->tabWidget->setCurrentIndex(0);

    // Mise en place du timer pour les relevés cycliques
    CycleTimer = new QTimer(this);
    connect(CycleTimer, SIGNAL(timeout()), this, SLOT(CycleTimerHandler()));
    CycleTimer->stop();

    // récupération des premières infos, c'est là que sera détecté un mauvais mot de passe
    // on en profite pour regarder s'il il a un argument envoyé à l'application lors du lancement
    // si on a "-r" alors on lance directement le cycle d'acquisition
    // cette option permet de mettre LiveBoxMonitor en démarrage automatique avec le PC
    if (AlgoType==0)
        LiveBoxGetInfos(&LBoxInfos);
    else
        LiveBox4GetInfos(&LBoxInfos);


    // Lecture du fichier CSV s'il existe
    if (ui->checkReloadCSV->isChecked()) ReadCSV();

    //LastInfos=LBoxInfos;
    if (IsConnected) {
        on_BtnActualiser_clicked();
        WinAccueil->close();
        QStringList args=QCoreApplication::arguments();
        //qDebug() << "args.count = " << args.count();
        for (int i=0 ; i<args.count() ; i++) {
            //qDebug() << "argument = " << args[i];
            if (args[i]=="-r") on_BtnStartAcquisition_clicked();
        }
    }
    else {
        WinAccueil->close();
        //WinPass->show();
        QMessageBox::warning(this,"LiveBox Monitoring","Connexion impossible, vérifer le mot de passe et le fonctionnement de la Box.");
        ui->tabWidget->setCurrentIndex(PARAMETRES); // on affiche l'onglet avec les paramètres
    }

}

MainWindow::~MainWindow()
{
    // sauvegarde des paramètres à la fermeture
    // position et mise en forme de la fenêtre principale
    QSettings settings(QApplication::applicationDirPath() + "/livebox.ini", QSettings::IniFormat);
    settings.setValue("LIVEBOX/SSID",LBoxInfos.SSID());
    settings.setValue("LIVEBOX/Pass",Pass);
    settings.setValue("GUI/Width",this->geometry().width());
    settings.setValue("GUI/Height",this->geometry().height());
    settings.setValue("GUI/X",this->geometry().x());
    settings.setValue("GUI/Y",this->geometry().y());
    settings.setValue("MONITOR/Cycle",ui->timeEditCycle->time().toString("HH:mm:ss"));
    settings.setValue("MONITOR/Arret",ui->timeEditArret->time().toString("HH:mm:ss"));
    settings.setValue("MONITOR/Diapo",ui->timeEditDiapo->time().toString("HH:mm:ss"));
    settings.setValue("MONITOR/ExportFormat",ui->comboExportFormat->currentIndex());
    settings.setValue("MONITOR/ReloadCSV",ui->checkReloadCSV->isChecked());
    settings.setValue("MONITOR/SaveAuto",ui->checkSaveAuto->isChecked());
    settings.setValue("MONITOR/RazAuto",ui->checkRazAuto->isChecked());
    settings.setValue("MONITOR/PdfAuto",ui->checkPdfAuto->isChecked());
    if (ui->radioLB23->isChecked())
        settings.setValue("LIVEBOX/AlgoType","0");
    else
        settings.setValue("LIVEBOX/AlgoType","1");

    // on libère la mémoire même si Qt doit le faire
    delete CycleTimer;
    delete WinAccueil;
    delete ui;
}

void MainWindow::closeEvent(QCloseEvent *)
{
    // on force la fermeture et la destruction de la fenetre "graphiques"
    // l'état de la fenetre "Graphiques" est sauvegardé lors de son propre évènement "Close"
    Graphiques->close();
    delete Graphiques;
}

void MainWindow::CycleTimerHandler()
{
    QDateTime Deb=QDateTime::currentDateTime();
    CycleTimer->stop();
    UpDateAffichage();
    QDateTime Fin=QDateTime::currentDateTime();
    quint64 Tps=Fin.toMSecsSinceEpoch()-Deb.toMSecsSinceEpoch();    // temps de traitement en milli secondes
    quint64 Tempo=Cycle-Tps;                                        // on enlève le temps de traitement
    if (Tempo<=0) Tempo=1000;                                       // on laisse au moins 1 seconde

    CycleTimer->start(Tempo);                                       // on envoie la tempo corigée

    if (!IsConnected) QMessageBox::warning(this,"LiveBox Monitoring","Problème d'acquisition !!!"); // pour informer n'arrête pas les cycles

    if (Tps>=Cycle) Cycle=Tps+1000;                                 // si Cycle était trop court, on compense
}

void MainWindow::RebootLiveBox()
{
    QNetworkRequest reqAuth;
    QNetworkRequest req;
    QNetworkReply  *reply;
    QString         strReply;
    QByteArray      postData;
    QJsonDocument   jsonResponse;
    QJsonObject     jsonObj;
    QByteArray      baContextID;
    QByteArray      baCookies;

    // Question pour confirmer le reboot
    int rep=QMessageBox::question(this,"Reboot LiveBox","Voulez-vous vraiement redémarrer votre LiveBox ?",QMessageBox::Yes | QMessageBox::No);
    if (rep==QMessageBox::No) {return;}

    // create custom temporary event loop on stack
    QEventLoop eventLoop;

    // "quit()" the event-loop, when the network request "finished()"
    //QNetworkAccessManager mgr;
    QObject::connect(&mgr, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));

    // ======== indentification sur la LiveBox et retour de ContextID
    QString    sPARAMSlogin;
    QByteArray PARAMSlogin;
    sPARAMSlogin = "{\"service\":\"sah.Device.Information\",\"method\":\"createContext\",\"parameters\":{\"applicationName\":\"so_sdkut\",\"username\":\"";
    sPARAMSlogin = sPARAMSlogin+User+"\",\"password\":\""+Pass+"\"}}";
    PARAMSlogin=sPARAMSlogin.toLatin1();
    reqAuth.setUrl(QUrl("http://"+LiveBox+"/ws"));
    reqAuth.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    reqAuth.setRawHeader("Authorization", "X-Sah-Login");
    postData=PARAMSlogin;
    reply = mgr.post(reqAuth,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonObj = jsonResponse.object();
        baContextID.append(jsonObj["data"].toObject()["contextID"].toString());
        baCookies=reply->rawHeader("Set-Cookie");
        IsConnected=true;
    }
    else {
        //failure
        QMessageBox::warning(this,"Authentification", "LB non connectée");
        qDebug() << "Failure" <<reply->errorString();
        IsConnected=false;
        return;
    }
    // ======== Envoi de la commande de reboot
    req.setUrl(QUrl("http://"+LiveBox+"/ws"));
    req.setRawHeader("Content-Type", "application/json");
    req.setRawHeader("X-Context", baContextID);
    req.setRawHeader("cookie", baCookies + "; sah/contextId=" + baContextID);
    postData="http://192.168.1.1/sysbus/NMC:reboot";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
    }
    else {
        //failure
        qDebug() << "Failure REBOOT" <<reply->errorString();
        return;
    }

}

// version pour LiveBox2 et Livebox3
void MainWindow::LiveBoxGetInfos(LiveBoxInfos *LBinfos)
{
    QNetworkRequest req;
    QNetworkReply  *reply;
    QString         strReply;
    QByteArray      postData;
    QJsonDocument   jsonResponse;
    QJsonObject     jsonObj;
    QJsonObject     jsonMIBs;
    QJsonObject     jsonPPP;
    QJsonObject     jsonDSLS;
    QJsonObject     jsonWIFI;
    QJsonObject     jsonWAMS;
    QJsonObject     jsonDevice;
    QByteArray      baContextID;

    // create custom temporary event loop on stack
    QEventLoop eventLoop;

    // "quit()" the event-loop, when the network request "finished()"
    //QNetworkAccessManager mgr;
    QObject::connect(&mgr, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));

    // ======== indentification sur la LiveBox et retour de ContextID
    req.setUrl(QUrl("http://"+LiveBox+"/authenticate?username="+User+"&password="+Pass));
    req.setRawHeader("Content-Type", "application/json");
    postData="";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonObj = jsonResponse.object();
        baContextID.append(jsonObj["data"].toObject()["contextID"].toString());
        IsConnected=true;
    }
    else {
        //failure
        qDebug() << "Failure" <<reply->errorString();
        IsConnected=false;
        return;
    }

    // ======== Informations json GetMIBs
    req.setUrl(QUrl("http://"+LiveBox+URLgetMIBs));
    req.setRawHeader("X-Context", baContextID);
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    postData=PARAMSgetMIBs;
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonMIBs = jsonResponse.object()["result"].toObject()["status"].toObject()["dsl"].toObject()["dsl0"].toObject();
    }
    else {
        //failure
        qDebug() << "Failure" <<reply->errorString();
        return;
    }

    // ======== Informations json GetPPP
    req.setUrl(QUrl("http://"+LiveBox+URLgetPPP));
    req.setRawHeader("X-Context", baContextID);
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    postData=PARAMSgetPPP;
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonPPP = jsonResponse.object()["result"].toObject()["status"].toObject()["ppp"].toObject()["ppp_data"].toObject();
    }
    else {
        //failure
        qDebug() << "Failure" <<reply->errorString();
        return;
    }

    // ======== Informations json DSLStats
    req.setUrl(QUrl("http://"+LiveBox+URLgetDSLStats));
    req.setRawHeader("X-Context", baContextID);
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    postData=PARAMSgetDSLStats;
    reply = mgr.post(req,postData);
    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonDSLS = jsonResponse.object()["result"].toObject()["status"].toObject();
    }
    else {
        //failure
        qDebug() << "Failure" <<reply->errorString();
        return;
    }

    // ======== Informations json WIFI
    req.setUrl(QUrl("http://"+LiveBox+URLgetWIFI));
    req.setRawHeader("X-Context", baContextID);
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    postData=PARAMSgetWififlags;
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonWIFI = jsonResponse.object()["result"].toObject()["status"].toObject()["wlanvap"].toObject()["wl0"].toObject();
    }
    else {
        //failure
        qDebug() << "Failure" <<reply->errorString();
        return;
    }

    // ======== Informations json GetWANstatus
    req.setUrl(QUrl("http://"+LiveBox+URLgetWANstatus));
    req.setRawHeader("X-Context", baContextID);
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    postData=PARAMSgetWANstatus;
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonWAMS = jsonResponse.object()["result"].toObject()["data"].toObject();
    }
    else {
        //failure
        qDebug() << "Failure" <<reply->errorString();
        return;
    }

    // ======== Informations json DeviceInfo
    req.setUrl(QUrl("http://"+LiveBox+URLgetDevice));
    req.setRawHeader("X-Context", baContextID);
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    postData="";
    reply = mgr.get(req);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonDevice = jsonResponse.object();
    }
    else {
        //failure
        qDebug() << "Failure" <<reply->errorString();
        return;
    }

    // ======== Extraction des informations
    LBinfos->setLastChange(jsonMIBs["LastChange"].toDouble());
    LBinfos->setLastChangeTime(jsonMIBs["LastChangeTime"].toDouble());
    LBinfos->setDownstreamAttenuation(jsonMIBs["DownstreamAttenuation"].toDouble());
    LBinfos->setDownstreamLineAttenuation(jsonMIBs["DownstreamLineAttenuation"].toDouble());
    LBinfos->setDownstreamCurrRate(jsonMIBs["DownstreamCurrRate"].toDouble());
    LBinfos->setDownstreamMaxRate(jsonMIBs["DownstreamMaxRate"].toDouble());
    LBinfos->setDownstreamNoiseMargin(jsonMIBs["DownstreamNoiseMargin"].toDouble());
    LBinfos->setDownstreamPower(jsonMIBs["DownstreamPower"].toDouble());
    LBinfos->setUpstreamAttenuation(jsonMIBs["UpstreamAttenuation"].toDouble());
    LBinfos->setUpstreamLineAttenuation(jsonMIBs["UpstreamLineAttenuation"].toDouble());
    LBinfos->setUpstreamCurrRate(jsonMIBs["UpstreamCurrRate"].toDouble());
    LBinfos->setUpstreamMaxRate(jsonMIBs["UpstreamMaxRate"].toDouble());
    LBinfos->setUpstreamNoiseMargin(jsonMIBs["UpstreamNoiseMargin"].toDouble());
    LBinfos->setUpstreamPower(jsonMIBs["UpstreamPower"].toDouble());
    LBinfos->setModulationType(jsonMIBs["ModulationType"].toString());
    LBinfos->setModulationHint(jsonMIBs["ModulationHint"].toString());
    LBinfos->setErroredSecs(jsonDSLS["ErroredSecs"].toDouble());
    LBinfos->setSeverelyErroredSecs(jsonDSLS["SeverelyErroredSecs"].toDouble());
    LBinfos->setATUCCRCErrors(jsonDSLS["ATUCCRCErrors"].toDouble());
    LBinfos->setCRCErrors(jsonDSLS["CRCErrors"].toDouble());
    LBinfos->setATUCFECErrors(jsonDSLS["ATUCFECErrors"].toDouble());
    LBinfos->setFECErrors(jsonDSLS["FECErrors"].toDouble());
    LBinfos->setATUCHECErrors(jsonDSLS["ATUCHECErrors"].toDouble());
    LBinfos->setHECErrors(jsonDSLS["HECErrors"].toDouble());
    LBinfos->setCellDelin(jsonDSLS["CellDelin"].toDouble());
    LBinfos->setInitErrors(jsonDSLS["InitErrors"].toDouble());
    LBinfos->setInitTimeouts(jsonDSLS["InitTimeouts"].toDouble());
    LBinfos->setLinkRetrain(jsonDSLS["LinkRetrain"].toDouble());
    LBinfos->setLossOfFraming(jsonDSLS["LossOfFraming"].toDouble());
    LBinfos->setReceiveBlocks(jsonDSLS["ReceiveBlocks"].toDouble());
    LBinfos->setTransmitBlocks(jsonDSLS["TransmitBlocks"].toDouble());
    LBinfos->setSSID(jsonWIFI["SSID"].toString());
    LBinfos->setPppLastChange(jsonPPP["LastChange"].toDouble());
    LBinfos->setPppLastChangeTime(jsonPPP["LastChangeTime"].toDouble());
    //QStringList ListDNS=jsonPPP["DNSServers"].toString().split(",");
    QStringList ListDNS=jsonWAMS["DNSServers"].toString().split(",");
    if (ListDNS.count()>0) LBinfos->setDNSServer1(ListDNS[0]);
    if (ListDNS.count()>1) LBinfos->setDNSServer2(ListDNS[1]);
    LBinfos->setTransportType(jsonPPP["TransportType"].toString());
    LBinfos->setMaxMRUSize(jsonPPP["MaxMRUSize"].toDouble());

    LBinfos->setDateTime(QDateTime::currentDateTime().toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setRebootTime(QDateTime::currentDateTime().addSecs(-LBinfos->LastChangeTime()-LBinfos->LastChange()).toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setSynchroTime(QDateTime::currentDateTime().addSecs(-LBinfos->LastChange()).toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setConnectTime(QDateTime::currentDateTime().addSecs(-LBinfos->PppLastChange()).toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setNumberOfReboots(jsonDevice["parameters"].toArray()[23].toObject()["value"].toDouble());
    LBinfos->setSofwareVersion(jsonDevice["parameters"].toArray()[7].toObject()["value"].toString());
    LBinfos->setHardwareVersion(jsonDevice["parameters"].toArray()[6].toObject()["value"].toString());
    LBinfos->setIPAddress(jsonWAMS["IPAddress"].toString());

    delete reply; // plus besoin
}

// Version pour Livebox 4 (et Livebox 3)
void MainWindow::LiveBox4GetInfos(LiveBoxInfos *LBinfos)
{
    QNetworkRequest reqAuth;
    QNetworkRequest req;
    QNetworkReply  *reply;
    QString         strReply;
    QByteArray      postData;
    QJsonDocument   jsonResponse;
    QJsonObject     jsonObj;
    QJsonObject     jsonMIBs;
    QJsonObject     jsonPPP;
    QJsonObject     jsonDSLS;
    QJsonObject     jsonWIFI;
    QJsonObject     jsonWAMS;
    QJsonObject     jsonMTU;
    QJsonObject     jsonDevice;
    QByteArray      baContextID;
    QByteArray      baCookies;

    // fichier info de debug
    QString FileName;
    FileName=QApplication::applicationDirPath()+"/debug.txt";
    QFile fichier(FileName);
    if (!fichier.open(QIODevice::WriteOnly | QIODevice::Text))
           return;
    QTextStream debug(&fichier);
    debug<<"Version LB4 02/06/19 - 19h40\n\n";

    // create custom temporary event loop on stack
    QEventLoop eventLoop;

    // "quit()" the event-loop, when the network request "finished()"
    QObject::connect(&mgr, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));

    //QMessageBox::information(this,"Info debug","Début algorithme LB4.");

    // ======== indentification sur la LiveBox et retour de ContextID
    QString    sPARAMSlogin;
    QByteArray PARAMSlogin;
    sPARAMSlogin = "{\"service\":\"sah.Device.Information\",\"method\":\"createContext\",\"parameters\":{\"applicationName\":\"so_sdkut\",\"username\":\"";
    sPARAMSlogin = sPARAMSlogin+User+"\",\"password\":\""+Pass+"\"}}";
    PARAMSlogin=sPARAMSlogin.toLatin1();
    reqAuth.setUrl(QUrl("http://"+LiveBox+"/ws"));
    reqAuth.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    reqAuth.setRawHeader("Authorization", "X-Sah-Login");
    postData=PARAMSlogin;
    reply = mgr.post(reqAuth,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonObj = jsonResponse.object();
        baContextID.append(jsonObj["data"].toObject()["contextID"].toString());
        baCookies=reply->rawHeader("Set-Cookie");
        IsConnected=true;
    }
    else {
        //failure
        QMessageBox::warning(this,"Authentification", "LB non connectée");
        qDebug() << "Failure" <<reply->errorString();
        IsConnected=false;
        return;
    }

    // ======== Informations json GetMIBS
    req.setUrl(QUrl("http://"+LiveBox+"/ws"));
    req.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    req.setRawHeader("X-Context", baContextID);
    req.setRawHeader("cookie", baCookies + "; sah/contextId=" + baContextID);
    postData="{\"service\":\"NeMo.Intf.data\",\"method\":\"getMIBs\",\"parameters\":{}}";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        //jsonMIBs = jsonResponse.object()["result"].toObject()["status"].toObject()["dsl"].toObject()["dsl0"].toObject();
        jsonMIBs = jsonResponse.object()["status"].toObject()["dsl"].toObject()["dsl0"].toObject();
        jsonMTU = jsonResponse.object()["status"].toObject()["netdev"].toObject()["atm_multi"].toObject();
        debug<<"\n--GetMIBS--"<<"\n"<<strReply<<"\n";
        //debug<<"jsonMIBS="<<jsonMIBs<<"\n";
    }
    else {
        //failure
        qDebug() << "Failure GetMIBS" <<reply->errorString();
        debug<<"failure GetMIBS"<<reply->errorString()<<"\n";
        return;
    }

    /* à priori pas de GetPPP sur LB4... les infos sont ailleurs
    // ======== Informations json GetPPP
    req.setUrl(QUrl("http://"+LiveBox+"/ws"));
    req.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    req.setRawHeader("X-Context", baContextID);
    req.setRawHeader("cookie", baCookies + "; sah/contextId=" + baContextID);
    postData="{\"service\":\"NeMo.Intf.data\",\"method\":\"getMIBs\",\"parameters\":{}}";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        jsonPPP = jsonResponse.object()["result"].toObject()["status"].toObject()["ppp"].toObject()["ppp_data"].toObject();
        debug<<"\n--GetPPP--"<<"\n"<<strReply<<"\n";
        debug<<"jsonPPP="<<jsonPPP<<"\n";
    }
    else {
        //failure
        qDebug() << "Failure GetPPP" <<reply->errorString();
        debug<<"failure GetPPP"<<reply->errorString()<<"\n";
        return;
    }
    */

    // ======== Informations json DSLstat
    req.setUrl(QUrl("http://"+LiveBox+"/ws"));
    req.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    req.setRawHeader("X-Context", baContextID);
    req.setRawHeader("cookie", baCookies + "; sah/contextId=" + baContextID);
    postData="{\"service\":\"NeMo.Intf.dsl0\",\"method\":\"getDSLStats\",\"parameters\":{}}";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        //jsonDSLS = jsonResponse.object()["result"].toObject()["status"].toObject();
        jsonDSLS = jsonResponse.object()["status"].toObject();
        debug<<"\n--DSLstat--"<<"\n"<<strReply<<"\n";
        //debug<<"jsonDSLS="<<jsonDSLS<<"\n";
    }
    else {
        //failure
        qDebug() << "Failure DSLstat" <<reply->errorString();
        debug<<"failure DSLstat"<<reply->errorString()<<"\n";
        return;
    }

    // ======== Informations json WIFI
    // {\"service\":\"NeMo.Intf.lan\",\"method\":\"getMIBs\",\"parameters\":{\"mibs\":\"wlanvap || wlanradio\"}}
    req.setUrl(QUrl("http://"+LiveBox+"/ws"));
    req.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    req.setRawHeader("X-Context", baContextID);
    req.setRawHeader("cookie", baCookies + "; sah/contextId=" + baContextID);
    postData="{\"service\":\"NeMo.Intf.lan\",\"method\":\"getMIBs\",\"parameters\":{\"mibs\":\"wlanvap || wlanradio\"}}";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        //jsonWIFI = jsonResponse.object()["result"].toObject()["status"].toObject()["wlanvap"].toObject()["wl0"].toObject();
        jsonWIFI = jsonResponse.object()["status"].toObject()["wlanvap"].toObject()["wl0"].toObject();
        debug<<"\n--WIFI--"<<"\n"<<strReply<<"\n";
        //debug<<"jsonWIFI="<<jsonWIFI<<"\n";
    }
    else {
        //failure
        qDebug() << "Failure WIFI" <<reply->errorString();
        debug<<"failure WIFI"<<reply->errorString()<<"\n";
        return;
    }

    // ======== Informations json WANstatus
    req.setUrl(QUrl("http://"+LiveBox+"/ws"));
    req.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    req.setRawHeader("X-Context", baContextID);
    req.setRawHeader("cookie", baCookies + "; sah/contextId=" + baContextID);
    postData="{\"service\":\"NMC\",\"method\":\"getWANStatus\",\"parameters\":{}}";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        //jsonWAMS = jsonResponse.object()["result"].toObject()["data"].toObject();
        jsonWAMS = jsonResponse.object()["data"].toObject();
        debug<<"\n--WANstatus--"<<"\n"<<strReply<<"\n";
        //debug<<"jsonWAMS="<<jsonWAMS<<"\n";
    }
    else {
        //failure
        qDebug() << "Failure WANstatus" <<reply->errorString();
        debug<<"failure WANstatus"<<reply->errorString()<<"\n";
        return;
    }

    // ======== Informations json DeviceInfo
    req.setUrl(QUrl("http://"+LiveBox+"/ws"));
    req.setRawHeader("Content-Type", "application/x-sah-ws-4-call+json");
    req.setRawHeader("X-Context", baContextID);
    req.setRawHeader("cookie", baCookies + "; sah/contextId=" + baContextID);
    postData="{\"service\":\"DeviceInfo\",\"method\":\"get\",\"parameters\":{}}";
    reply = mgr.post(req,postData);

    eventLoop.exec(); // blocks stack until "finished()" has been called

    if (reply->error() == QNetworkReply::NoError) {
        //success
        strReply = (QString)reply->readAll();
        jsonResponse = QJsonDocument::fromJson(strReply.toUtf8());
        //jsonDevice = jsonResponse.object();
        jsonDevice = jsonResponse.object()["status"].toObject();
        debug<<"\n--DeviceInfo--"<<"\n"<<strReply<<"\n";
        //debug<<"jsonDevice="<<jsonDevice<<"\n";
    }
    else {
        //failure
        qDebug() << "Failure DeviceInfo" <<reply->errorString();
        debug<<"failure DeviceInfo"<<reply->errorString()<<"\n";
        return;
    }

    debug<<"\nEXTRACTION DES INFOS\n\n";

    // ======== Extraction des informations
    LBinfos->setLastChange(jsonMIBs["LastChange"].toDouble());
    LBinfos->setLastChangeTime(jsonMIBs["LastChangeTime"].toDouble());
    LBinfos->setDownstreamAttenuation(jsonMIBs["DownstreamAttenuation"].toDouble());
    LBinfos->setDownstreamLineAttenuation(jsonMIBs["DownstreamLineAttenuation"].toDouble());
    LBinfos->setDownstreamCurrRate(jsonMIBs["DownstreamCurrRate"].toDouble());
    LBinfos->setDownstreamMaxRate(jsonMIBs["DownstreamMaxRate"].toDouble());
    LBinfos->setDownstreamNoiseMargin(jsonMIBs["DownstreamNoiseMargin"].toDouble());
    LBinfos->setDownstreamPower(jsonMIBs["DownstreamPower"].toDouble());
    LBinfos->setUpstreamAttenuation(jsonMIBs["UpstreamAttenuation"].toDouble());
    LBinfos->setUpstreamLineAttenuation(jsonMIBs["UpstreamLineAttenuation"].toDouble());
    LBinfos->setUpstreamCurrRate(jsonMIBs["UpstreamCurrRate"].toDouble());
    LBinfos->setUpstreamMaxRate(jsonMIBs["UpstreamMaxRate"].toDouble());
    LBinfos->setUpstreamNoiseMargin(jsonMIBs["UpstreamNoiseMargin"].toDouble());
    LBinfos->setUpstreamPower(jsonMIBs["UpstreamPower"].toDouble());
    LBinfos->setModulationType(jsonMIBs["ModulationType"].toString());
    LBinfos->setModulationHint(jsonMIBs["ModulationHint"].toString());
    LBinfos->setErroredSecs(jsonDSLS["ErroredSecs"].toDouble());
    LBinfos->setSeverelyErroredSecs(jsonDSLS["SeverelyErroredSecs"].toDouble());
    LBinfos->setATUCCRCErrors(jsonDSLS["ATUCCRCErrors"].toDouble());
    LBinfos->setCRCErrors(jsonDSLS["CRCErrors"].toDouble());
    LBinfos->setATUCFECErrors(jsonDSLS["ATUCFECErrors"].toDouble());
    LBinfos->setFECErrors(jsonDSLS["FECErrors"].toDouble());
    LBinfos->setATUCHECErrors(jsonDSLS["ATUCHECErrors"].toDouble());
    LBinfos->setHECErrors(jsonDSLS["HECErrors"].toDouble());
    LBinfos->setCellDelin(jsonDSLS["CellDelin"].toDouble());
    LBinfos->setInitErrors(jsonDSLS["InitErrors"].toDouble());
    LBinfos->setInitTimeouts(jsonDSLS["InitTimeouts"].toDouble());
    LBinfos->setLinkRetrain(jsonDSLS["LinkRetrain"].toDouble());
    LBinfos->setLossOfFraming(jsonDSLS["LossOfFraming"].toDouble());
    LBinfos->setReceiveBlocks(jsonDSLS["ReceiveBlocks"].toDouble());
    LBinfos->setTransmitBlocks(jsonDSLS["TransmitBlocks"].toDouble());
    LBinfos->setSSID(jsonWIFI["SSID"].toString());
//    LBinfos->setPppLastChange(jsonPPP["LastChange"].toDouble());
//    LBinfos->setPppLastChangeTime(jsonPPP["LastChangeTime"].toDouble());
    //QStringList ListDNS=jsonPPP["DNSServers"].toString().split(",");
    QStringList ListDNS=jsonWAMS["DNSServers"].toString().split(",");
    if (ListDNS.count()>0) LBinfos->setDNSServer1(ListDNS[0]);
    if (ListDNS.count()>1) LBinfos->setDNSServer2(ListDNS[1]);
    LBinfos->setTransportType(jsonWAMS["Protocol"].toString());
    LBinfos->setMaxMRUSize(jsonMTU["MTU"].toDouble());

    LBinfos->setDateTime(QDateTime::currentDateTime().toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setRebootTime(QDateTime::currentDateTime().addSecs(-LBinfos->LastChangeTime()-LBinfos->LastChange()).toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setSynchroTime(QDateTime::currentDateTime().addSecs(-LBinfos->LastChange()).toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setConnectTime(QDateTime::currentDateTime().addSecs(-LBinfos->PppLastChange()).toString("dd/MM/yyyy hh:mm:ss"));
    LBinfos->setNumberOfReboots(jsonDevice["NumberOfReboots"].toDouble());
    LBinfos->setSofwareVersion(jsonDevice["SoftwareVersion"].toString());
    LBinfos->setHardwareVersion(jsonDevice["HardwareVersion"].toString());
    LBinfos->setIPAddress(jsonWAMS["IPAddress"].toString());

    delete reply; // plus besoin

    debug<<"hardware version = "<<LBinfos->HardwareVersion()<<"\n";
    debug<<"LastChange = "<<QString::number(LBinfos->LastChange())<<"\n";

    fichier.close(); // fermeture du fichier de debug
}

void MainWindow::AppendCSV()
{
    QString FileName;
    FileName=QApplication::applicationDirPath()+"/"+LBoxInfos.SSID()+".csv";
    //qDebug() << FileName;

    // Test pour sauvegarde automatique au changement de jour
    // tester si LBoxInfo.DateTime donne le même jour que LastInfo.DateTime
    // si sauvegarde auto activée, faire la sauvegarde avant de renseigner le CSV
    // si le RAZ après sauvegarde est activé lancer CreateCSV
    QDateTime tLast=QDateTime::fromString(LastInfos.DateTime(),"dd/MM/yyyy hh:mm:ss");
    QDateTime tInfo=QDateTime::fromString(LBoxInfos.DateTime(),"dd/MM/yyyy hh:mm:ss");
    //qDebug() << "Last Day" << tLast.date().day() << tLast.time().hour()<<"Info day" << tInfo.date().day()<< tInfo.time().hour();
    if ( tLast.date().day() != tInfo.date().day() ) {                       // on test s'il y a changement de jour entre les deux derniers relevés
    //if ( tLast.time().hour() != tInfo.time().hour() ) {                   // test idem mais toutes les heures, utile en débug.....
        // sauvegarde
        if (ui->checkSaveAuto->isChecked()) {                               // si sauvegarde cochée, on sauve le CSV.
            QString FileSave;
            FileSave=QApplication::applicationDirPath()+"/archives/"+LBoxInfos.SSID()+tLast.toString("_yyyyMMdd_HHmmss")+"auto.csv";
            SaveCSV(FileSave);
        }
        // rapport pDF
        if (ui->checkPdfAuto->isChecked()) {
            Graphiques->SetZoomTout();
            if (Graphiques->getPlageHeure()>24) Graphiques->SetZoomPlage(24); // on limite le graphique à 24h
            QString FilePDF;
            FilePDF=QApplication::applicationDirPath()+"/export/"+LBoxInfos.SSID()+"_Rapport"+tLast.toString("_yyyyMMdd_HHmmss")+"auto.pdf";
            QString Day;
            Day = tLast.date().toString("dd/MM/yyyy");
            Graphiques->RapportGraphique(FilePDF,Day);
            Graphiques->SetZoom3H();
        }
        // RAZ
        if (ui->checkRazAuto->isChecked()) {
            CreateCSV();                                                    // si RAZ coché, on recrée l'entête CSV sans sauver.
            Graphiques->ClearCurves();                                      // puis on efface les courbes
        }
    }



    QFile fichier(FileName);
    if (!fichier.open(QIODevice::Append | QIODevice::Text))
           return;
    QTextStream out(&fichier);
    out << LBoxInfos.DateTime() << ";";
    out << QString::number(LBoxInfos.LastChangeTime(),20,0) << ";";
    out << QString::number(LBoxInfos.LastChange(),20,0) << ";";
    out << QString::number(LBoxInfos.UpstreamCurrRate(),20,0) << ";";
    out << QString::number(LBoxInfos.DownstreamCurrRate(),20,0) << ";";
    out << QString::number(LBoxInfos.UpstreamMaxRate(),20,0) << ";";
    out << QString::number(LBoxInfos.DownstreamMaxRate(),20,0) << ";";
    out << QString::number(LBoxInfos.UpstreamNoiseMargin(),20,0) << ";";
    out << QString::number(LBoxInfos.DownstreamNoiseMargin(),20,0) << ";";
    out << QString::number(LBoxInfos.UpstreamLineAttenuation(),20,0) << ";";
    out << QString::number(LBoxInfos.DownstreamLineAttenuation(),20,0) << ";";
    out << QString::number(LBoxInfos.UpstreamPower(),20,0) << ";";
    out << QString::number(LBoxInfos.DownstreamPower(),20,0) << ";";
    out << QString::number(LBoxInfos.ReceiveBlocks(),20,0) << ";";
    out << QString::number(LBoxInfos.TransmitBlocks(),20,0) << ";";
    out << QString::number(LBoxInfos.CellDelin(),20,0) << ";";
    out << QString::number(LBoxInfos.LinkRetrain(),20,0) << ";";
    out << QString::number(LBoxInfos.InitErrors(),20,0) << ";";
    out << QString::number(LBoxInfos.InitTimeouts(),20,0) << ";";
    out << QString::number(LBoxInfos.LossOfFraming(),20,0) << ";";
    out << QString::number(LBoxInfos.ErroredSecs(),20,0) << ";";
    out << QString::number(LBoxInfos.SeverelyErroredSecs(),20,0) << ";";
    out << QString::number(LBoxInfos.FECErrors(),20,0) << ";";
    out << QString::number(LBoxInfos.ATUCFECErrors(),20,0) << ";";
    out << QString::number(LBoxInfos.HECErrors(),20,0) << ";";
    out << QString::number(LBoxInfos.ATUCHECErrors(),20,0) << ";";
    out << QString::number(LBoxInfos.CRCErrors(),20,0) << ";";
    out << QString::number(LBoxInfos.ATUCCRCErrors(),20,0) << ";";
    out << LBoxInfos.IPAddress() << ";";
    out << QString::number(LBoxInfos.PppLastChangeTime(),20,0) << ";";
    out << QString::number(LBoxInfos.PppLastChange(),20,0);
    out << QString::number(LBoxInfos.UpstreamAttenuation(),20,0) << ";";
    out << QString::number(LBoxInfos.DownstreamAttenuation(),20,0) << ";";
    out << "\n";
    fichier.close();

    // stockage des synchronisations
    if ( (LastChangeTime != LBoxInfos.LastChangeTime())|(PppLastChangeTime != LBoxInfos.PppLastChangeTime()) ) {
        qDebug() << LastChangeTime << LBoxInfos.PppLastChangeTime();
        LastChangeTime=LBoxInfos.LastChangeTime();
        PppLastChangeTime=LBoxInfos.PppLastChangeTime();
        LastSyncho=LBoxInfos.SynchroTime();
        QString FileNameSynchro;
        FileNameSynchro=QApplication::applicationDirPath()+"/"+LBoxInfos.SSID()+"_SYNCHROS.csv";

        QFile fichierSynchro(FileNameSynchro);
        if (!fichierSynchro.exists()) {
            if (fichierSynchro.open(QIODevice::WriteOnly | QIODevice::Text)) {
                QTextStream out(&fichierSynchro);
                out << "DATE HEURE SYNCHRO ;DATE HEURE RELEVE  ; SESSION DE SUIVI  ; DEMARRAGE BOX     ; CONNEXION INTERNET";
                out << "\n";
                //     "dd/MM/yyyy hh:mm:ss;dd/MM/yyyy hh:mm:ss;dd/MM/yyyy hh:mm:ss;dd/MM/yyyy hh:mm:ss;dd/MM/yyyy hh:mm:ss"
                fichierSynchro.close();
                Graphiques->UpDateSynchronisations();
            }
            else {
                return;
            }
        }
        if (!fichierSynchro.open(QIODevice::Append | QIODevice::Text))
               return;
        QTextStream out(&fichierSynchro);
        out << LastSyncho << ";" << LBoxInfos.DateTime() << ";" << SessionDateTime << ";" ;
        out << LBoxInfos.RebootTime()  << ";" << LBoxInfos.ConnectTime() << "\n";
        fichierSynchro.close();
    }
}

void MainWindow::CreateCSV()
{
    QString FileName;
    FileName=QApplication::applicationDirPath()+"/"+LBoxInfos.SSID()+".csv";

    QFile fichier(FileName);
    if (!fichier.open(QIODevice::WriteOnly | QIODevice::Text))
           return;
    QTextStream out(&fichier);
    out << "DateTime;LastChangeTime;LastChange;UpstreamCurrRate;DownstreamCurrRate;UpstreamMaxRate;DownstreamMaxRate;UpstreamNoiseMargin;";
    out << "DownstreamNoiseMargin;UpstreamLineAttenuation;DownstreamLineAttenuation;UpstreamPower;DownstreamPower;ReceiveBlocks;TransmitBlocks;";
    out << "CellDelin;LinkRetrain;InitErrors;InitTimeouts;LossOfFraming;ErroredSecs;SeverelyErroredSecs;";
    out << "FECErrors;ATUCFECErrors;HECErrors;ATUCHECErrors;CRCErrors;ATUCCRCErrors;IPAddress;PppLastChangeTime;PppLastChange;UpstreamAttenuation;DownstreamAttenuation";
    out << "\n";
    fichier.close();
}

void MainWindow::ReadCSV()
{
    QString FileName;
    FileName=QApplication::applicationDirPath()+"/"+SSID+".csv";
    QString     ligne;
    QFile       fichier(FileName);
    bool        ligne1=true;
    if(fichier.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QTextStream flux(&fichier);
        if (!flux.atEnd()) ligne = flux.readLine(); // pour sauter la ligne d'entête
        while(!flux.atEnd())
        {
            LastInfos=LBoxInfos;
            ligne = flux.readLine();
            QStringList listLigne=ligne.split(";");
            LBoxInfos.setDateTime(listLigne[0]);
            LBoxInfos.setLastChangeTime(listLigne[1].toDouble());
            LBoxInfos.setLastChange(listLigne[2].toDouble());
            LBoxInfos.setUpstreamCurrRate(listLigne[3].toDouble());
            LBoxInfos.setDownstreamCurrRate(listLigne[4].toDouble());
            LBoxInfos.setUpstreamMaxRate(listLigne[5].toDouble());
            LBoxInfos.setDownstreamMaxRate(listLigne[6].toDouble());
            LBoxInfos.setUpstreamNoiseMargin(listLigne[7].toDouble());
            LBoxInfos.setDownstreamNoiseMargin(listLigne[8].toDouble());
            LBoxInfos.setUpstreamLineAttenuation(listLigne[9].toDouble());
            LBoxInfos.setDownstreamLineAttenuation(listLigne[10].toDouble());
            LBoxInfos.setUpstreamPower(listLigne[11].toDouble());
            LBoxInfos.setDownstreamPower(listLigne[12].toDouble());
            LBoxInfos.setReceiveBlocks(listLigne[13].toDouble());
            LBoxInfos.setTransmitBlocks(listLigne[14].toDouble());
            LBoxInfos.setCellDelin(listLigne[15].toDouble());
            LBoxInfos.setLinkRetrain(listLigne[16].toDouble());
            LBoxInfos.setInitErrors(listLigne[17].toDouble());
            LBoxInfos.setInitTimeouts(listLigne[18].toDouble());
            LBoxInfos.setLossOfFraming(listLigne[19].toDouble());
            LBoxInfos.setErroredSecs(listLigne[20].toDouble());
            LBoxInfos.setSeverelyErroredSecs(listLigne[21].toDouble());
            LBoxInfos.setFECErrors(listLigne[22].toDouble());
            LBoxInfos.setATUCFECErrors(listLigne[23].toDouble());
            LBoxInfos.setHECErrors(listLigne[24].toDouble());
            LBoxInfos.setATUCHECErrors(listLigne[25].toDouble());
            LBoxInfos.setCRCErrors(listLigne[26].toDouble());
            LBoxInfos.setATUCCRCErrors(listLigne[27].toDouble());
            LBoxInfos.setIPAddress(listLigne[28]);
            if (listLigne.count()>29) LBoxInfos.setPppLastChangeTime(listLigne[29].toDouble());
            if (listLigne.count()>30) LBoxInfos.setPppLastChange(listLigne[30].toDouble());
            if (listLigne.count()>31) LBoxInfos.setUpstreamAttenuation(listLigne[31].toDouble());
            if (listLigne.count()>32) LBoxInfos.setDownstreamAttenuation(listLigne[32].toDouble());
            if (!ligne1) Graphiques->UpDateCurves(&LBoxInfos,&LastInfos);
            ligne1=false;
        }
        fichier.close();
    }
}

void MainWindow::SaveCSV(QString FileSave)
{
    QString FileName;
    FileName=QApplication::applicationDirPath()+"/"+LBoxInfos.SSID()+".csv";
    QFile Fichier(FileName);
    Fichier.copy(FileSave);
}

void MainWindow::UpDateAffichage()
{
    this->setCursor(Qt::WaitCursor);
    LastInfos=LBoxInfos;
    if (AlgoType==0)
        LiveBoxGetInfos(&LBoxInfos);
    else
        LiveBox4GetInfos(&LBoxInfos);

    Graphiques->setSSID(LBoxInfos.SSID());
    Graphiques->setHardwareVersion(LBoxInfos.HardwareVersion());
    Graphiques->setSoftwareVersion(LBoxInfos.SoftwareVersion());

    QDateTime Synchro=QDateTime::fromString(LBoxInfos.SynchroTime(),"dd/MM/yyyy hh:mm:ss");
    QDateTime Now=QDateTime::fromString(LBoxInfos.DateTime(),"dd/MM/yyyy hh:mm:ss");
    uint DureeSynchro=Now.toTime_t()-Synchro.toTime_t(); // durée de synchro exprimée en secondes
    uint SynchJours=DureeSynchro/86400;
    uint SynchHeures=(DureeSynchro-86400*SynchJours)/3600;
    uint SynchMinutes=(DureeSynchro-86400*SynchJours-3600*SynchHeures)/60;
    uint SynchSecondes=DureeSynchro-86400*SynchJours-3600*SynchHeures-60*SynchMinutes;
    QString SynchText;
    SynchText=QString("%1Jr %2H %3mn %4s").arg(SynchJours,2,'f',0,'0').arg(SynchHeures,2,'f',0,'0').arg(SynchMinutes,2,'f',0,'0').arg(SynchSecondes,2,'f',0,'0');

    ui->editRebootTime->setText(LBoxInfos.RebootTime());
    ui->editSynchroTime->setText(LBoxInfos.SynchroTime());
    ui->editSynchroDuree->setText(SynchText);
    ui->editDateTime->setText(LBoxInfos.DateTime());
    ui->editDateTime_2->setText(LBoxInfos.DateTime());
    ui->editDateTime_3->setText(LBoxInfos.DateTime());
    ui->editUpStreamCurrRate->setText(QString::number(LBoxInfos.UpstreamCurrRate(),20,0)+" Kb/s");
    ui->editDownStreamCurrRate->setText(QString::number(LBoxInfos.DownstreamCurrRate(),20,0)+" Kb/s");
    ui->editUpStreamMaxRate->setText(QString::number(LBoxInfos.UpstreamMaxRate(),20,0)+" Kb/s");
    ui->editDownStreamMaxRate->setText(QString::number(LBoxInfos.DownstreamMaxRate(),20,0)+" Kb/s");
    ui->editUpStreamUseRate->setText(QString::number(0.85*LBoxInfos.UpstreamCurrRate()/1000,20,2)+" Mb/s");
    ui->editDownStreamUseRate->setText(QString::number(0.85*LBoxInfos.DownstreamCurrRate()/1000,20,2)+" Mb/s");
    ui->editIPAdress->setText(LBoxInfos.IPAddress());
    ui->editNumberOfReboot->setText(QString::number(LBoxInfos.NumberOfReboots(),20,0));
    ui->editSSID->setText(LBoxInfos.SSID());
    ui->editModulationType->setText(LBoxInfos.ModulationType());
    ui->editModulationHint->setText(LBoxInfos.ModulationHint());
    ui->editSoftwareVersion->setText(LBoxInfos.SoftwareVersion());
    ui->editHardwareVersion->setText(LBoxInfos.HardwareVersion());

    ui->editConnectTime->setText(LBoxInfos.ConnectTime());
    ui->editDNS1->setText(LBoxInfos.DNSServer1());
    ui->editDNS2->setText(LBoxInfos.DNSServer2());
    ui->editTransportType->setText(LBoxInfos.TransportType());
    ui->editMaxMRUSize->setText(QString::number(LBoxInfos.MaxMRUSize(),20,0));

    ui->editLinkRetran->setText(QString::number(LBoxInfos.LinkRetrain(),20,0));
    ui->editInitErrors->setText(QString::number(LBoxInfos.InitErrors(),20,0));
    ui->editFECErrors->setText(QString::number(LBoxInfos.FECErrors(),20,0));
    ui->editATUCFECErrors->setText(QString::number(LBoxInfos.ATUCFECErrors(),20,0));
    ui->editHECErrors->setText(QString::number(LBoxInfos.HECErrors(),20,0));
    ui->editATUCHECErrors->setText(QString::number(LBoxInfos.ATUCHECErrors(),20,0));
    ui->editCRCErrors->setText(QString::number(LBoxInfos.CRCErrors(),20,0));
    ui->editATUCCRCErrors->setText(QString::number(LBoxInfos.ATUCCRCErrors(),20,0));
    ui->editUpStreamNoiseMargin->setText(QString::number(LBoxInfos.UpstreamNoiseMargin()/10,20,1)+" dB");
    ui->editDownStreamNoiseMargin->setText(QString::number(LBoxInfos.DownstreamNoiseMargin()/10,20,1)+" dB");
    ui->editUpStreamAttenuation->setText(QString::number(LBoxInfos.UpstreamLineAttenuation()/10,20,1)+" dB");
    ui->editDownStreamAttenuation->setText(QString::number(LBoxInfos.DownstreamLineAttenuation()/10,20,1)+" dB");
    ui->editErroredSec->setText(QString::number(LBoxInfos.ErroredSecs(),20,0)+" s");
    ui->editSeverelyErroredSec->setText(QString::number(LBoxInfos.SeverelyErroredSecs(),20,0)+" s");

    ui->editFECDelta->setText(QString::number(LBoxInfos.FECErrors()-LastInfos.FECErrors(),20,0));
    ui->editATUCFECDelta->setText(QString::number(LBoxInfos.ATUCFECErrors()-LastInfos.ATUCFECErrors(),20,0));
    ui->editHECDelta->setText(QString::number(LBoxInfos.HECErrors()-LastInfos.HECErrors(),20,0));
    ui->editATUCHECDelta->setText(QString::number(LBoxInfos.ATUCHECErrors()-LastInfos.ATUCHECErrors(),20,0));
    ui->editCRCDelta->setText(QString::number(LBoxInfos.CRCErrors()-LastInfos.CRCErrors(),20,0));
    ui->editATUCCRCDelta->setText(QString::number(LBoxInfos.ATUCCRCErrors()-LastInfos.ATUCCRCErrors(),20,0));
    ui->editUpStreamNoiseDelta->setText(QString::number((LBoxInfos.UpstreamNoiseMargin()-LastInfos.UpstreamNoiseMargin())/10,20,1)+" dB");
    ui->editDownStreamNoiseDelta->setText(QString::number((LBoxInfos.DownstreamNoiseMargin()-LastInfos.DownstreamNoiseMargin())/10,20,1)+" dB");
    ui->editErroredSecDelta->setText(QString::number(LBoxInfos.ErroredSecs()-LastInfos.ErroredSecs(),20,0)+" s");
    ui->editSeverelyErroredSecDelta->setText(QString::number(LBoxInfos.SeverelyErroredSecs()-LastInfos.SeverelyErroredSecs(),20,0)+" s");

    AppendCSV();

    Graphiques->UpDateCurves(&LBoxInfos, &LastInfos);

    this->setCursor(Qt::ArrowCursor);
}

void MainWindow::on_BtnGraphiques_clicked()
{
    Graphiques->show();
}

void MainWindow::on_BtnQuitter_clicked()
{
    this->close();
}

void MainWindow::on_checkPass_clicked(bool checked)
{
    // affichage ou nom du mot de passe en clair
    if (checked)
        ui->editPass->setEchoMode(QLineEdit::Normal);
    else
        ui->editPass->setEchoMode(QLineEdit::Password);
}

void MainWindow::on_tabWidget_currentChanged()
{
    // l'affichage du mot de passe doit être une action volontaire
    // donc on masque par défaut à chaque changement d'onglet
    ui->editPass->setEchoMode(QLineEdit::Password);
    ui->checkPass->setChecked(false);
}

void MainWindow::on_timeEditDiapo_timeChanged(const QTime &time)
{
    Diapo=3600*time.hour()+60*time.minute()+time.second();
    Diapo=1000*Diapo;  // le cycle dans un QTimer doit être donné en milli secondes.
    Graphiques->setDiapo(Diapo);
}

void MainWindow::on_BtnActualiser_clicked()
{
    ui->BtnActualiser->setEnabled(false);
    ui->BtnGraphiques->setEnabled(true);
    ui->BtnStartAcquisition->setEnabled(false);
    ui->BtnStopAcquisition->setEnabled(false);
    ui->BtnSaveReleve->setEnabled(false);
    ui->BtnRazReleve->setEnabled(false);
    ui->BtnQuitter->setEnabled(false);
    UpDateAffichage();
    ui->BtnActualiser->setEnabled(true);
    ui->BtnGraphiques->setEnabled(true);
    ui->BtnStartAcquisition->setEnabled(true);
    ui->BtnStopAcquisition->setEnabled(false);
    ui->BtnSaveReleve->setEnabled(true);
    ui->BtnRazReleve->setEnabled(true);
    ui->BtnQuitter->setEnabled(true);
}

void MainWindow::on_BtnStartAcquisition_clicked()
{
    ui->BtnActualiser->setEnabled(false);
    ui->BtnGraphiques->setEnabled(true);
    ui->BtnStartAcquisition->setEnabled(false);
    ui->BtnStopAcquisition->setEnabled(true);
    ui->BtnSaveReleve->setEnabled(false);
    ui->BtnRazReleve->setEnabled(false);
    ui->BtnQuitter->setEnabled(false);

    ui->statusBar->showMessage("SUIVI LIVEBOX EN COURS...");
    CycleTimer->start(Cycle);
}

void MainWindow::on_BtnStopAcquisition_clicked()
{
    ui->BtnActualiser->setEnabled(true);
    ui->BtnGraphiques->setEnabled(true);
    ui->BtnStartAcquisition->setEnabled(true);
    ui->BtnStopAcquisition->setEnabled(false);
    ui->BtnSaveReleve->setEnabled(true);
    ui->BtnRazReleve->setEnabled(true);
    ui->BtnQuitter->setEnabled(true);

    ui->statusBar->showMessage("");
    CycleTimer->stop();
}

void MainWindow::on_BtnSaveReleve_clicked()
{
    QString FileSave;
    FileSave=QApplication::applicationDirPath()+"/archives/"+LBoxInfos.SSID()+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+".csv";
    SaveCSV(FileSave);
}

void MainWindow::on_BtnRazReleve_clicked()
{
    int ret=QMessageBox::question(this,"Remise à zéro","Voulez-vous sauvegarder avant la remise à zéro ?");

    if (ret==QMessageBox::Yes) {
        QString FileSave;
        FileSave=QApplication::applicationDirPath()+"/archives/"+LBoxInfos.SSID()+QDateTime::currentDateTime().toString("_yyyyMMdd_HHmmss")+".csv";
        SaveCSV(FileSave);
    }
    CreateCSV();
    Graphiques->ClearCurves();
}

void MainWindow::on_editPass_textChanged(const QString &arg1)
{
    Pass=arg1;
}


void MainWindow::on_timeEditCycle_timeChanged(const QTime &time)
{
    Cycle=3600*time.hour()+60*time.minute()+time.second();
    if (Cycle<5) {
        Cycle=5;
        ui->timeEditCycle->setTime(QTime(0,0,5,0));
    }
    Cycle=1000*Cycle;               // le cycle dans un QTimer doit être donné en milli secondes.
    qDebug() << Cycle;

    // vérification et correction éventuelle de la détection de temps d'arrêt à 2 x cycle
    QTime tCycle=QTime::fromString(ui->timeEditCycle->text(),"hh:mm:ss");
    int sCycle=3600*tCycle.hour()+60*tCycle.minute()+tCycle.second();
    QTime tArret=QTime::fromString(ui->timeEditArret->text(),"hh:mm:ss");
    int sArret=3600*tArret.hour()+60*tArret.minute()+tArret.second();
    if ((2*sCycle)>sArret) {
        qDebug() << "correction temps arret" << sArret;
        sArret=2*sCycle;
        QTime tArret;
        tArret.setHMS(0,0,0,0);
        tArret=tArret.addSecs(sArret);
        qDebug() << tArret;
        ui->timeEditArret->setTime(tArret);
        Graphiques->setTempsArret(sArret);
    }
}

void MainWindow::on_timeEditArret_timeChanged(const QTime &time)
{
    QTime tCycle=QTime::fromString(ui->timeEditCycle->text(),"hh:mm:ss");
    int sCycle=3600*tCycle.hour()+60*tCycle.minute()+tCycle.second();
    int sArret=3600*time.hour()+60*time.minute()+time.second();
    qDebug() << "EditArret" << sCycle << tCycle;
    if ((2*sCycle)>sArret) {
        qDebug() << "correction temps arret" << sArret;
        sArret=2*sCycle;
        QTime tArret;
        tArret.setHMS(0,0,0,0);
        tArret=tArret.addSecs(sArret);
        qDebug() << tArret;
        ui->timeEditArret->setTime(tArret);
    }
    Graphiques->setTempsArret(sArret);
}

void MainWindow::on_comboExportFormat_currentIndexChanged(const QString &arg1)
{
    Graphiques->setExportFormat(arg1);
}

void MainWindow::on_checkDiapoCourbes_toggled(bool checked)
{
    Graphiques->setDiapoCourbes(checked);
}

void MainWindow::on_checkDiapoErreurs_toggled(bool checked)
{
    Graphiques->setDiapoErreurs(checked);
}

void MainWindow::on_BtnRapportGraphique_clicked()
{
    Graphiques->RapportGraphique("","");
}

void MainWindow::on_checkAffDate_toggled(bool checked)
{
    Graphiques->setEchDate(checked);
    Graphiques->UpDateScales();
}

void MainWindow::on_checkAffHeure_toggled(bool checked)
{
    Graphiques->setEchHeure(checked);
    Graphiques->UpDateScales();
}

void MainWindow::on_checkPortrait_toggled()
{
    Graphiques->setRapPageOrient(QPrinter::Portrait);
}

void MainWindow::on_checkPaysage_toggled()
{
    Graphiques->setRapPageOrient(QPrinter::Landscape);
}

void MainWindow::on_btnReboot_clicked()
{
    RebootLiveBox();
}
